﻿/**************************************************************************
 * GisTest.cs
 * GIS名前空間に属するクラス群をテストする静的クラス for C#
 * 
 * 開発者　：森下功啓（K.Morhista Kumamoto-University）
 * 開発履歴：
 *          2012/5/19   この頃、開発開始
 *          2012/5/28   ここ4日ほどを使って、SaveDemFileInfoAsKml()，SaveMapAsBmp(), SaveBigMapAsBmp(), ReadNmeaToRectangleField()を新設した。
 * ***********************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using GIS.DigitalElevationModel;
using GIS.DigitalElevationModel.Reader;
using GIS.MapBase;
using GIS.KML.Polygon;
using GNSS;
using GNSS.Field;
using GNSS.GPS.NMEA;


namespace GIS
{
    /// <summary>
    /// テスト専用の静的クラス
    /// </summary>
    public static class GisTest
    {
        /// <summary>
        /// フォルダを指定して、走査されたDEMファイルの一覧をKMLファイルへ出力する
        /// <para>ReaderクラスとManagerクラスの動作テストです。</para>
        /// <para>ファイル名はDEMmaps.kmlです。</para>
        /// <para>異種類のDEMは先読みされた方を優先して処理し、混合されませんのでご注意ください。</para>
        /// <example>
        /// <code>
        /// GIS.GisTest.SaveDemFileInfoAsKml(GIS.KML.Color.TransRed);
        /// </code>
        /// </example>
        /// </summary>
        /// <param name="color">色情報（format: AABBGGRR）</param>
        public static void SaveDemFileInfoAsKml(string color = KML.Color.TransBule)
        {
            Reader reader = new Reader();
            var hoge = reader.ScanDirWithDialog(ReadMode.HeaderOnly);
            Manager man = new Manager();
            man.Add(hoge);
            using (System.IO.StreamWriter fwriter = new System.IO.StreamWriter("DEMmaps.kml", false, Encoding.UTF8))
            {
                try
                {
                    fwriter.Write(man.ToStringAsKml(color));            // 文字列化したデータを保存する。
                }
                catch
                {
                    // エラー処理があれば記述
                }
            }
            return;
        }
        /// <summary>
        /// 読み込んだDEMファイルを一つだけビットマップファイルとして出力します
        /// <para>Readerクラスの読み込み機能及び、Mapクラスの画像出力機能のテストです。</para>
        /// <para>
        /// 出力するファイルは選べません。
        /// ファイルを全て読み込むので処理にかなりの時間がかかります。
        /// </para>
        /// <example>
        /// <code>
        /// GIS.GisTest.SaveMapAsBmp();
        /// </code>
        /// </example>
        /// </summary>
        public static void SaveMapAsBmp()
        {
            Reader reader = new Reader();
            var hoge = reader.ScanDirWithDialog(ReadMode.HeaderAndValue);
            if (hoge.Count != 0)
            {
                if (hoge[0].IsLoaded)
                {
                    DemSet demst = hoge[0];
                    demst.DemMap.SaveAsImage("ToBmpTest.bmp");
                }
            }
            return;
        }
        /// <summary>
        /// DEMクラスの読み込みテストおよびマネジメント機能のテスト、さらにマップの結合テストを実施します
        /// <para>指定した座標で張る長方形領域を含む地図を生成します。</para>
        /// <para>座標を省略すると、BLH(33.0432, 130.559), BLH(32.6159, 130.9659)（熊本県付近）が使用されます。</para>
        /// <para>Taskクラスを利用して処理がすぐに戻るようにしておりますが、実際には若干の間をおいてファイルが生成されることにご注意ください。</para>
        /// </summary>
        /// <param name="x1">座標1</param>
        /// <param name="x2">座標2</param>
        public static void SaveBigMapAsBmp(BLH x1 = new BLH(), BLH x2 = new BLH())
        {
            DEM dem = new DEM();
            try
            {
                dem.AddDirWithDialog();
                if (dem.Count != 0)
                {
                    Map map = null;
                    var tsk = Task.Factory.StartNew(() =>
                    {
                        if (x1 == new BLH() && x1 == x2)
                            map = dem.CreateMap(new RectangleField(new BLH(33.0432, 130.559), new BLH(32.6159, 130.9659)));
                        else
                            map = dem.CreateMap(new RectangleField(x1, x2));

                        //System.Diagnostics.Stopwatch sw2 = System.Diagnostics.Stopwatch.StartNew();

                        if (map != null) map.SaveAsImage("ToBmpTest.bmp");

                        //sw2.Stop();                                                         // ストップウォッチを止める
                        //Console.WriteLine("マップ画像出力にかけた処理時間: " + sw2.Elapsed); // 結果を表示する
                    });
                }
            }
            catch (SystemException e)
            {
                Console.WriteLine(e.Message);
            }
            return;
        }
        /// <summary>
        /// Polygonクラスのテスト
        /// <para>GNSSプロジェクトのGnssTestクラスの機能を利用してNMEAファイルからRectangleFieldオブジェクトを生成し、Polygonクラスを利用してKMLファイルを出力します。</para>
        /// <para>ダイアログを利用してファイルを開きます。</para>
        /// </summary>
        public static void ReadNmeaToRectangleField()
        {
            RectangleField field = GNSS.GnssTest.ReadNmeaToRectangleFeild();
            if (field.AreaIsZero == false)
            {
                Polygon polygon = new Polygon();
                polygon.AddToOuter(field.ToArray());                // KML出力に備え、座標をセット
                // ファイル出力
                // KML出力はUTF8で出力してください。
                using (System.IO.StreamWriter fwriter = new System.IO.StreamWriter("NMEAtoField.kml", false, Encoding.UTF8))
                {
                    try
                    {
                        fwriter.Write(polygon.CreateCompleteCode());
                    }
                    catch
                    {
                        // エラー処理があれば記述
                    }
                }
            }
            return;
        }
    }
}
