﻿/****************************************************************************
 * Polygon.cs
 * KMLのポリゴンを生成するクラス
 * 
 * [目的]
 *  ポリゴンの生成を目的としています。
 * 
 * [history]
 *      2012/5/24   開発開始
 *                  KMLの座標系はどうすんだ？
 * **************************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GNSS;

namespace GIS.KML.Polygon
{
    /// <summary>
    /// Polygonクラス
    /// <para>プロパティ名等はKMLに合わせています。</para>
    /// </summary>
    public class Polygon
    {
        /* メンバ変数 *********************************/
        /// <summary>
        /// 外側の点情報
        /// </summary>
        private List<BLH> outerBuff;
        /// <summary>
        /// 内側の点情報
        /// </summary>
        private List<BLH> innerBUff;
        /// <summary>
        /// ポリゴン
        /// </summary>
        private List<Polygon> polygons;
        /* プロパティ *********************************/
        /// <summary>
        /// 色
        /// </summary>
        public string color
        {
            get;
            set;
        }
        /// <summary>
        /// 名前
        /// </summary>
        public string name
        {
            get;
            set;
        }
        /* メソッド *********************************/
        /// <summary>
        /// 内側に点を追加する
        /// </summary>
        /// <param name="pos">位置座標</param>
        public void AddToInner(BLH pos)
        {
            this.innerBUff.Add(pos.ToDegree());
        }
        /// <summary>
        /// 内側に点を追加する
        /// </summary>
        /// <param name="pos">位置座標</param>
        public void AddToInner(BLH[] pos)
        {
            foreach (BLH position in pos)
            {
                this.innerBUff.Add(position.ToDegree());
            }
            return;
        }
        /// <summary>
        /// 外側へ点を追加する
        /// </summary>
        /// <param name="pos">位置座標</param>
        public void AddToOuter(BLH pos)
        {
            this.outerBuff.Add(pos.ToDegree());
        }
        /// <summary>
        /// 外側へ点を追加する
        /// </summary>
        /// <param name="pos">位置座標</param>
        public void AddToOuter(BLH[] pos)
        {
            foreach (BLH position in pos)
            {
                this.outerBuff.Add(position.ToDegree());
            }
            return;
        }
        /// <summary>
        /// ポリゴンを追加する
        /// <para>ポリゴンオブジェクトを内部のListに追加します。</para>
        /// <para>まとめて一つの文字列化することを念頭に入れています。</para>
        /// </summary>
        /// <param name="polygon"></param>
        public void AddPolygon(Polygon polygon)
        {
            this.polygons.Add(polygon);
        }
        /// <summary>
        /// 本インスタンスのスタイルコードを返す
        /// </summary>
        /// <returns>スタイルコード</returns>
        public string CreateStyleCode()
        {
            StringBuilder sb = new StringBuilder(300);                      // 予め大きなメモリ容量を確保しておく
            sb.Append(@"    <Style id=""colorPoly"">").Append(System.Environment.NewLine);
            sb.Append(@"      <LineStyle>").Append(System.Environment.NewLine);
            sb.Append(@"        <width>1.5</width>").Append(System.Environment.NewLine);
            sb.Append(@"      </LineStyle>").Append(System.Environment.NewLine);
            sb.Append(@"      <PolyStyle>").Append(System.Environment.NewLine);
            sb.Append(@"        <color>").Append(this.color).Append("</color>").Append(System.Environment.NewLine);
            sb.Append(@"      </PolyStyle>").Append(System.Environment.NewLine);
            sb.Append(@"    </Style>").Append(System.Environment.NewLine);
            return sb.ToString();
        }
        /// <summary>
        /// 本インスタンスのポリゴンのコア部分のみのコードを返す
        /// </summary>
        /// <returns>ポリゴンのコアコード</returns>
        public string CreatePolygonCode()
        {
            StringBuilder sb = new StringBuilder(1000);                      // 予め大きなメモリ容量を確保しておく
            sb.Append(@"    <Placemark>").Append(System.Environment.NewLine);
            sb.Append(@"      <styleUrl>#colorPoly</styleUrl>").Append(System.Environment.NewLine);
            sb.Append(@"      <name>").Append(this.name).Append("</name>").Append(System.Environment.NewLine);
            sb.Append(@"       <Polygon>
        <extrude>1</extrude>
        <altitudeMode>clampToGround</altitudeMode>
        <outerBoundaryIs>
          <LinearRing>
            <coordinates>").Append(System.Environment.NewLine); ;
            foreach (BLH pos in this.outerBuff)
            {
                sb.Append("              ").Append(pos.ToString()).Append(System.Environment.NewLine);
            }
            sb.Append(@"            </coordinates>
          </LinearRing>
        </outerBoundaryIs>").Append(System.Environment.NewLine); 
            sb.Append(@"      <innerBoundaryIs>
        <LinearRing>
          <coordinates>").Append(System.Environment.NewLine); 
            foreach (BLH pos in this.innerBUff)
            {
                sb.Append("              ").Append(pos.ToString()).Append(System.Environment.NewLine);
            }
            sb.Append(@"          </coordinates>
        </LinearRing>
      </innerBoundaryIs>").Append(System.Environment.NewLine);
            sb.Append(@"      </Polygon>").Append(System.Environment.NewLine);
            sb.Append(@"    </Placemark>").Append(System.Environment.NewLine);
            return sb.ToString();
        }
        /// <summary>
        /// 本インスタンスが保持するポリゴン成分のみで完全コードを生成して返します
        /// </summary>
        /// <returns>本オブジェクトで作れるポリゴンの完全コード</returns>
        public string CreateCompleteCode()
        {
            StringBuilder sb = new StringBuilder(1500);                      // 予め大きなメモリ容量を確保しておく
            sb.Append(@"<?xml version=""1.0"" encoding=""UTF-8""?>").Append(System.Environment.NewLine);
            sb.Append(@"<kml xmlns=""http://www.opengis.net/kml/2.2"">").Append(System.Environment.NewLine);
            sb.Append(@"  <Document>").Append(System.Environment.NewLine);
            sb.Append(this.CreateStyleCode());
            sb.Append(this.CreatePolygonCode());
            sb.Append(@"  </Document>").Append(System.Environment.NewLine);
            sb.Append(@"</kml>").Append(System.Environment.NewLine);
            return sb.ToString();
        }
        /// <summary>
        /// 格納されているポリゴンのオブジェクトに加えて本インスタンスの情報も加えて文字列出力します。
        /// </summary>
        /// <returns></returns>
        public string CreatePolygonsCode()
        {
            StringBuilder sb = new StringBuilder(1500 * (this.polygons.Count + 1));// 予め大きなメモリ容量を確保しておく
            sb.Append(@"<?xml version=""1.0"" encoding=""UTF-8""?>").Append(System.Environment.NewLine);
            sb.Append(@"<kml xmlns=""http://www.opengis.net/kml/2.2"">").Append(System.Environment.NewLine);
            sb.Append(@"  <Document>").Append(System.Environment.NewLine);
            sb.Append(this.CreateStyleCode());
            sb.Append(this.CreatePolygonCode());
            foreach (Polygon poly in this.polygons)
            {
                sb.Append(poly.CreatePolygonCode());
            }
            sb.Append(@"  </Document>").Append(System.Environment.NewLine);
            sb.Append(@"</kml>").Append(System.Environment.NewLine);
            return sb.ToString();
        }
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public Polygon()
            :base()
        {
            this.name = "";
            this.color = "7dff0000";            // 半透明青
            this.outerBuff = new List<BLH>();
            this.innerBUff = new List<BLH>();
            this.polygons = new List<Polygon>();
        }
    }
}
